# YAESU_FT8_FREQ_CONTROLLER.pyw
# 
# programed by Toshi JA0RUG

import tkinter as tk
import tkinter.messagebox as txtmsg
import socket
import time

# メインウインドウ
root= tk.Tk()
root.title('YAESU FT8 FREQ CONTROL')
root.option_add('*Button.font', ('', 12, 'bold'))
root.geometry("280x460") 
root.configure(bg = "skyblue")

# サブウインドウ
sub_window = tk.Toplevel()
sub_window.title('YAESU FT8 FREQ CONTROL')
sub_window.geometry("300x160")
sub_window.protocol('WM_DELETE_WINDOW', (lambda: 'pass')())

# グローバル変数
ip   = "127.0.0.1" # デフォルト値
port = 5555        # デフォルト値

# グローバル変数 複数の関数で使用
connect_flag = False

# メッセージ表示用変数
msg  = tk.StringVar()

# 終了
def exit1():

    if txtmsg.askokcancel('','終了しますか？'):

        if connect_flag == True:
            # ソケットクローズ
            sock.close()
        print("終了しました.")
        exit()

# ファイルに書き込まれたIPアドレスとポートを読み込み, 値を返す.
# 読み込み失敗の場合、default値を使う
def read_ip_setting(ip_addr, port_num):

    # ファイルが存在していたらデータを読み込む. 読み込めない場合や
    # ip, portの2つのデータが無い場合はdefault値を使う

    try:

        f = open("ip_setting.txt", "r")
        line= f.readline().rstrip('\n')
        print("設定ファイルから設定値を読み込みました.")

        data = line.split()  

        if len(data) < 2:
            print("設定ファイルのデータが足りません. 設定に初期値を使います.")
            ip_addr, port_num = ip, port

        else:
            ip_addr   = data[0] 
            port_num = data[1]

        f.close()

    # 設定ファイルが存在しなければ, default値を使う
    except OSError as e:
        print(e)
        print("ip_setting.txtがありません.\n")
        print("設定に初期値を使います.")


    print("接続先のIPアドレス/ポートは,", ip_addr, port_num)
    return (ip_addr, port_num)

def get_freq():

    freq = entry_freq.get()

    freq_len = len(freq)

    if  freq.isdigit() and (freq_len == 5 or freq_len == 4):
        msg.set("")
        if freq_len == 4:
            freq = "0" + freq

        set_freq(freq+"000")

    else:
        msg.set("周波数は,4桁または5桁の整数です.")

    entry_freq.delete(0, tk.END) 

def connect():

    global sock, connect_flag

    print("1:", connect_flag)

    if connect_flag == True: # 接続済みの場合は, 何もせずリターン
        return

    msg.set("")

    ip = entry_ip.get()

    try:
        socket.inet_aton(ip)

    except socket.error:
        msg.set("IPアドレスが不正です.")
        return

    port = entry_port.get()    

    if port.isdigit() == False:
        msg.set("PORT番号が不正です.")
        return

    # サーバーに接続
    try: 
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.settimeout(1.0) # タイムアウト値 単位は秒
        sock.connect((ip, int(port)))

        entry_ip.configure(fg="white", bg="green")
        entry_port.configure(fg="white", bg="green")
        msg.set("")
        connect_flag = True
        print("接続しました.")
        print("接続先:", ip, "PORT:", int(port))

    except socket.error as e:

        entry_ip.configure(fg="white", bg="red")
        entry_port.configure(fg="white", bg="red")
        connect_flag = False
        msg.set("接続に失敗しました.")

    print("2:", connect_flag)

def disconnect():
    global connect_flag

    if connect_flag == True:
        # 送受信の切断
        sock.shutdown(socket.SHUT_RDWR)
        # ソケットクローズ
        sock.close()

        connect_flag = False

        msg.set("")
        print("切断しました.")

        entry_ip.configure(fg="black", bg="white")
        entry_port.configure(fg="black", bg="white")

    print("3:", connect_flag)

def save():
    ip_addr = entry_ip.get()
    port_num = entry_port.get()

    print("ip_setting.txtに設定を保存しました.", ip_addr, port_num)

    f = open("ip_setting.txt", 'w')
    f.write(ip_addr  + " " +  port_num +'\n')
    f.close()

def set_freq(data):

    global connect_flag

    msg.set("")

    try:
        sock.send(b"MD02;")
        print("send: MD02;")
        time.sleep(0.1)

        sock.send(("FA" + data + ";").encode())    # 無線機(A)用ルーチン

        print("send:", "FA Command")

    # soketが切断されていた場合
    except:
        msg.set("切断されています. 接続ボタンを押してください.")
        connect_flag = False
        entry_ip.configure(fg="white", bg="red")
        entry_port.configure(fg="white", bg="red")

root.protocol('WM_DELETE_WINDOW', exit1)

print("YAESU FT8 周波数設定クライアント")

# 接続設定読み込み
(ip, port) = read_ip_setting(ip, port)

# メインウインドウ側設定
frame1 = tk.Frame(root, bg = "skyblue")
frame1.pack(side = tk.LEFT, padx =10)
frame2 = tk.Frame(root, bg = "skyblue")
frame2.pack(side = tk.LEFT, padx =10)

label00 = tk.Label(frame1, text= "FT8", font=("", 12, "bold"), fg="white", bg="blue", width = 10).pack(padx=5, pady=5, fill=tk.X)
button00=tk.Button(frame1, text="1840", width = 10, command = lambda: set_freq("01840000")).pack(padx=5, pady=2, fill=tk.X)
button01=tk.Button(frame1, text="3573",  command = lambda: set_freq("03573000")).pack(padx=5, pady=2, fill=tk.X)
button02=tk.Button(frame1, text="7074",  command = lambda: set_freq("07074000")).pack(padx=5, pady=2, fill=tk.X)
button03=tk.Button(frame1, text="10136", command = lambda: set_freq("10136000")).pack(padx=5, pady=2, fill=tk.X)
button04=tk.Button(frame1, text="14074", command = lambda: set_freq("14074000")).pack(padx=5, pady=2, fill=tk.X)
button05=tk.Button(frame1, text="18100", command = lambda: set_freq("18100000")).pack(padx=5, pady=2, fill=tk.X)
button06=tk.Button(frame1, text="21074", command = lambda: set_freq("21074000")).pack(padx=5, pady=2, fill=tk.X)
button07=tk.Button(frame1, text="24915", command = lambda: set_freq("24915000")).pack(padx=5, pady=2, fill=tk.X)
button08=tk.Button(frame1, text="28074", command = lambda: set_freq("28074000")).pack(padx=5, pady=2, fill=tk.X)
button09=tk.Button(frame1, text="50313", command = lambda: set_freq("50313000")).pack(padx=5, pady=2, fill=tk.X)
button0a=tk.Button(frame1, text="50323", command = lambda: set_freq("50323000")).pack(padx=5, pady=2, fill=tk.X)

label10 = tk.Label(frame2, text = "FT4",  font=("", 12, "bold"), fg="white", bg="SlateBlue1", width = 10).pack(padx=5, pady=5, fill=tk.X)
button10=tk.Button(frame2, text="3568", width = 10, command = lambda: set_freq("03568000")).pack(padx=5, pady=2, fill=tk.X)
button11=tk.Button(frame2, text="3575",  command = lambda: set_freq("03575000")).pack(padx=5, pady=2, fill=tk.X)
button12=tk.Button(frame2, text="7047.5", command = lambda: set_freq("07047500")).pack(padx=5, pady=2, fill=tk.X)
button13=tk.Button(frame2, text="10140", command = lambda: set_freq("10140000")).pack(padx=5, pady=2, fill=tk.X)
button14=tk.Button(frame2, text="14080", command = lambda: set_freq("14080000")).pack(padx=5, pady=2, fill=tk.X)
button15=tk.Button(frame2, text="18104", command = lambda: set_freq("18104000")).pack(padx=5, pady=2, fill=tk.X)
button16=tk.Button(frame2, text="21140", command = lambda: set_freq("21140000")).pack(padx=5, pady=2, fill=tk.X)
button17=tk.Button(frame2, text="24919", command = lambda: set_freq("24919000")).pack(padx=5, pady=2, fill=tk.X)
button18=tk.Button(frame2, text="28180", command = lambda: set_freq("28180000")).pack(padx=5, pady=2, fill=tk.X)
button19=tk.Button(frame2, text="50318", command = lambda: set_freq("50318000")).pack(padx=5, pady=2, fill=tk.X)
button1a=tk.Button(frame2, text="     ", command = lambda: set_freq("")).pack(padx=5, pady=2, fill=tk.X)

# サブウインドウ側設定
frame_1 = tk.Frame(sub_window)
frame_1.pack(pady=5)

frame_2 = tk.Frame(sub_window) # 接続など
frame_2.pack()

frame_3 = tk.Frame(sub_window) # 情報表示
frame_3.pack()

frame_4 = tk.Frame(sub_window, bg="blue") # FREQ設定
frame_4.pack()

# 'IP' 文字表示 frame 1
label_A = tk.Label(frame_1, text='IP').pack(side = tk.LEFT, pady=2)

# IP Adress表示 frame 1
entry_ip = tk.Entry(frame_1, width=15, font=("", 10, "bold"), justify=tk.CENTER)
entry_ip.insert(0, ip)
entry_ip.pack(side = tk.LEFT, pady=10)

# 'PORT' 文字表示 frame 1
label_B = tk.Label(frame_1, text='     PORT').pack(side = tk.LEFT, pady=2)

# PORT 表示 frame 1
entry_port = tk.Entry(frame_1, width=5, font=("", 10, "bold"), justify=tk.CENTER)
entry_port.insert(0, port)
entry_port.pack(side = tk.LEFT, pady=5)

# 保存ボタン frame 2
tk.Button(frame_2, text = "設定保存", font=("", 10, "bold"), command = lambda: save()).pack(side = tk.LEFT, padx=10) # 保存ボタン

# 接続ボタン frame 2
tk.Button(frame_2, text = " 接続 ", font=("", 10, "bold"), command = lambda: connect()).pack(side = tk.LEFT, padx=10) # 接続ボタン

# 切断ボタン frame 2
tk.Button(frame_2, text = " 切断 ", font=("", 10, "bold"), command = lambda: disconnect()).pack(side = tk.LEFT, padx=10) # 接続ボタン


# 情報表示 frame_3
label_C = tk.Label(frame_3, textvariable = msg, fg="red").pack(side = tk.LEFT, pady=5, padx=10)

# "FREQ" 文字表示
label_D = tk.Label(frame_4, text='FREQ', font=("", 10, "bold"), fg="white", bg="blue").pack(side = tk.LEFT, padx=10)

# 周波数表示
entry_freq = tk.Entry(frame_4, width=8, font=("", 14, "bold"))
entry_freq.pack(side = tk.LEFT, padx=2, pady=5)

# "Set Freq" 文字表示
button=tk.Button(frame_4, text= ' Set Freq ', font=("", 8, "bold"), justify=tk.CENTER, command=get_freq).pack(side = tk.LEFT, padx=20, pady=5)

root.mainloop()
